/*!
        @file    pyb_correlator.cpp

        @brief   python binding of correlator measurement classes

        @author  Tatsumi Aoyama (aoym)
                 $LastChangedBy: aoyama $

        @date    $LastChangedDate:: 2021-02-25 17:32:18 #$

        @version $LastChangedRevision: 2177 $
*/

#include "pyb_defs.h"
#include "pyb_params.h"
#include "pyb_correlator.h"
#include "bridge.h"
#include "Field/field.h"
#include "Measurements/Fermion/corr2pt_4spinor.h"
#include "Measurements/Fermion/corr4pt_4spinor.h"
#include "Tools/gammaMatrixSet.h"
#include "Parameters/parameters.h"
#include "pyb_wrap.h"

void pybind_correlator(py::module m)
{

  //! 2pt Correlator
  //
  //  parameters:
  //    filename_output (string)
  //

  py::class_<Corr2pt_4spinor>(m, "Corr2pt_4spinor", R"pybdoc(
Two-point correlator for Wiilson-type fermions

Parameters
----------
filename_output : string
)pybdoc")
    .def(py::init([](py::object gmset, py::kwargs kwargs) -> Corr2pt_4spinor*
                  {
                    auto obj = new PyWrap1<Corr2pt_4spinor, GammaMatrixSet>(gmset);
                    if (obj) obj->set_parameters(make_parameters(kwargs));
                    return obj;
                  }), REDIRECT)
    
#define DEFINE_METHOD(x) def(#x, &Corr2pt_4spinor:: x, REDIRECT)
    .DEFINE_METHOD(meson_all)
    .DEFINE_METHOD(meson_correlator)
    .DEFINE_METHOD(meson_momentum_all)
    .DEFINE_METHOD(meson_momentum_correlator)

    .DEFINE_METHOD(meson_correlator_x)
    .DEFINE_METHOD(meson_momentum_correlator_x)

    .DEFINE_METHOD(proton_test)
    .DEFINE_METHOD(proton_correlator)

    .DEFINE_METHOD(proton_correlator_x)
#undef DEFINE_METHOD
    ;


  //! 4pt Correlator
  //
  //  parameters:
  //    filename_output (string)
  //

  py::class_<Corr4pt_4spinor>(m, "Corr4pt_4spinor", R"pybdoc(
Four-point correlator for Wiilson-type fermions

Parameters
----------
filename_output : string
)pybdoc")
    .def(py::init([](py::object gmset, py::kwargs kwargs) -> Corr4pt_4spinor*
                  {
                    auto obj = new PyWrap1<Corr4pt_4spinor, GammaMatrixSet>(gmset);
                    if (obj) obj->set_parameters(make_parameters(kwargs));
                    return obj;
                  }), REDIRECT)

#define DEFINE_METHOD(x) def(#x, &Corr4pt_4spinor:: x, REDIRECT)
    .DEFINE_METHOD(meson_all)
    .DEFINE_METHOD(meson_correlator)
    .DEFINE_METHOD(meson_momentum_all)
    .DEFINE_METHOD(meson_momentum_correlator)
#undef DEFINE_METHOD
    ;



}
